@@ -42,12 +42,14 @@
 #include "config.h"
 #endif
 
-#include "clutter-backend.h"
+#include "clutter-backend-private.h"
 #include "clutter-debug.h"
 #include "clutter-fixed.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
 #include "clutter-profile.h"
+#include "clutter-stage-manager-private.h"
+#include "clutter-stage-private.h"
 
 #include <cogl/cogl.h>
 
@@ -60,24 +62,19 @@ G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT);
 
 struct _ClutterBackendPrivate
 {
-  /* settings */
-  guint double_click_time;
-  guint double_click_distance;
-
-  gdouble resolution;
-
-  gfloat units_per_em;
-  gint32 units_serial;
-
   cairo_font_options_t *font_options;
 
   gchar *font_name;
+
+  gfloat units_per_em;
+  gint32 units_serial;
 };
 
 enum
 {
   RESOLUTION_CHANGED,
   FONT_CHANGED,
+  SETTINGS_CHANGED,
 
   LAST_SIGNAL
 };
@@ -87,7 +84,6 @@ static guint backend_signals[LAST_SIGNAL] = { 0, };
 static void
 clutter_backend_dispose (GObject *gobject)
 {
-  ClutterBackendPrivate *priv = CLUTTER_BACKEND (gobject)->priv;
   ClutterMainContext *clutter_context;
 
   clutter_context = _clutter_context_get_default ();
@@ -101,13 +97,20 @@ clutter_backend_dispose (GObject *gobject)
       clutter_context->events_queue = NULL;
     }
 
-  g_free (priv->font_name);
-
-  clutter_backend_set_font_options (CLUTTER_BACKEND (gobject), NULL);
-
   G_OBJECT_CLASS (clutter_backend_parent_class)->dispose (gobject);
 }
 
+static void
+clutter_backend_finalize (GObject *gobject)
+{
+  ClutterBackend *backend = CLUTTER_BACKEND (gobject);
+
+  g_free (backend->priv->font_name);
+  clutter_backend_set_font_options (backend, NULL);
+
+  G_OBJECT_CLASS (clutter_backend_parent_class)->finalize (gobject);
+}
+
 static gfloat
 get_units_per_em (ClutterBackend       *backend,
                   PangoFontDescription *font_desc)
@@ -120,12 +123,18 @@ get_units_per_em (ClutterBackend       *backend,
 
   if (font_desc == NULL)
     {
-      const gchar *font_name = clutter_backend_get_font_name (backend);
+      ClutterSettings *settings;
+      gchar *font_name = NULL;
+
+      settings = clutter_settings_get_default ();
+      g_object_get (settings, "font-name", &font_name, NULL);
 
       if (G_LIKELY (font_name != NULL && *font_name != '\0'))
         {
           font_desc = pango_font_description_from_string (font_name);
           free_font_desc = TRUE;
+
+          g_free (font_name);
         }
     }
 
@@ -166,11 +175,22 @@ static void
 clutter_backend_real_resolution_changed (ClutterBackend *backend)
 {
   ClutterBackendPrivate *priv = backend->priv;
+  ClutterMainContext *context;
+  ClutterSettings *settings;
+  gint dpi;
+
+  context = _clutter_context_get_default ();
+
+  settings = clutter_settings_get_default ();
+  g_object_get (settings, "font-dpi", &dpi, NULL);
+
+  if (context->font_map != NULL)
+    cogl_pango_font_map_set_resolution (context->font_map, dpi / 1024.0);
 
   priv->units_per_em = get_units_per_em (backend, NULL);
   priv->units_serial += 1;
 
-  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->priv->units_per_em);
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
 static void
@@ -181,7 +201,7 @@ clutter_backend_real_font_changed (ClutterBackend *backend)
   priv->units_per_em = get_units_per_em (backend, NULL);
   priv->units_serial += 1;
 
-  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", backend->priv->units_per_em);
+  CLUTTER_NOTE (BACKEND, "Units per em: %.2f", priv->units_per_em);
 }
 
 static void
@@ -190,6 +210,7 @@ clutter_backend_class_init (ClutterBackendClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->dispose = clutter_backend_dispose;
+  gobject_class->finalize = clutter_backend_finalize;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterBackendPrivate));
 
@@ -199,7 +220,7 @@ clutter_backend_class_init (ClutterBackendClass *klass)
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (ClutterBackendClass, resolution_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   backend_signals[FONT_CHANGED] =
@@ -208,7 +229,16 @@ clutter_backend_class_init (ClutterBackendClass *klass)
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (ClutterBackendClass, font_changed),
                   NULL, NULL,
-                  clutter_marshal_VOID__VOID,
+                  _clutter_marshal_VOID__VOID,
+                  G_TYPE_NONE, 0);
+
+  backend_signals[SETTINGS_CHANGED] =
+    g_signal_new (I_("settings-changed"),
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (ClutterBackendClass, settings_changed),
+                  NULL, NULL,
+                  _clutter_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 
   klass->resolution_changed = clutter_backend_real_resolution_changed;
@@ -222,8 +252,6 @@ clutter_backend_init (ClutterBackend *backend)
 
   priv = backend->priv = CLUTTER_BACKEND_GET_PRIVATE (backend);
 
-  priv->resolution = -1.0;
-
   priv->units_per_em = -1.0;
   priv->units_serial = 1;
 }
@@ -395,6 +423,17 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
           clutter_actor_get_size (CLUTTER_ACTOR (stage), &width, &height);
 
           _cogl_onscreen_clutter_backend_set_size (width, height);
+
+          /* Eventually we will have a separate CoglFramebuffer for
+           * each stage and each one will track private projection
+           * matrix and viewport state, but until then we need to make
+           * sure we update the projection and viewport whenever we
+           * switch between stages.
+           *
+           * This dirty mechanism will ensure they are asserted before
+           * the next paint... */
+          _clutter_stage_dirty_viewport (stage);
+          _clutter_stage_dirty_projection (stage);
         }
 
       /* FIXME: With a NULL stage and thus no active context it may make more
@@ -403,17 +442,6 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
        * potential issue of GL calls with no context)
        */
       current_context_stage = new_stage;
-
-      /* if the new stage has a different size than the previous one
-       * we need to update the viewport; we do it by simply setting the
-       * SYNC_MATRICES flag and letting the next redraw cycle take care
-       * of calling glViewport()
-       */
-      if (current_context_stage)
-        {
-          CLUTTER_SET_PRIVATE_FLAGS (current_context_stage,
-                                     CLUTTER_ACTOR_SYNC_MATRICES);
-        }
     }
   else
     CLUTTER_NOTE (MULTISTAGE, "Stage is the same");
@@ -498,6 +526,36 @@ _clutter_backend_get_units_per_em (ClutterBackend       *backend,
   return priv->units_per_em;
 }
 
+void
+_clutter_backend_copy_event_data (ClutterBackend     *backend,
+                                  const ClutterEvent *src,
+                                  ClutterEvent       *dest)
+{
+  ClutterBackendClass *klass;
+
+  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  g_return_if_fail (src != NULL);
+  g_return_if_fail (dest != NULL);
+
+  klass = CLUTTER_BACKEND_GET_CLASS (backend);
+  if (klass->copy_event_data != NULL)
+    klass->copy_event_data (backend, src, dest);
+}
+
+void
+_clutter_backend_free_event_data (ClutterBackend *backend,
+                                  ClutterEvent   *event)
+{
+  ClutterBackendClass *klass;
+
+  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  g_return_if_fail (event != NULL);
+
+  klass = CLUTTER_BACKEND_GET_CLASS (backend);
+  if (klass->free_event_data != NULL)
+    klass->free_event_data (backend, event);
+}
+
 /**
  * clutter_get_default_backend:
  *
@@ -531,14 +589,16 @@ clutter_get_default_backend (void)
  * verify whether it's a double click event or not.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-time instead
  */
 void
 clutter_backend_set_double_click_time (ClutterBackend *backend,
                                        guint           msec)
 {
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
+  ClutterSettings *settings = clutter_settings_get_default ();
 
-  backend->priv->double_click_time = msec;
+  g_object_set (settings, "double-click-time", msec, NULL);
 }
 
 /**
@@ -551,13 +611,18 @@ clutter_backend_set_double_click_time (ClutterBackend *backend,
  * Return value: a time in milliseconds
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-time instead
  */
 guint
 clutter_backend_get_double_click_time (ClutterBackend *backend)
 {
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
+  ClutterSettings *settings = clutter_settings_get_default ();
+  gint retval;
 
-  return backend->priv->double_click_time;
+  g_object_get (settings, "double-click-time", &retval, NULL);
+
+  return retval;
 }
 
 /**
@@ -568,14 +633,16 @@ clutter_backend_get_double_click_time (ClutterBackend *backend)
  * Sets the maximum distance used to verify a double click event.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-distance instead
  */
 void
 clutter_backend_set_double_click_distance (ClutterBackend *backend,
                                            guint           distance)
 {
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-  
-  backend->priv->double_click_distance = distance;
+  ClutterSettings *settings = clutter_settings_get_default ();
+
+  g_object_set (settings, "double-click-distance", distance, NULL);
 }
 
 /**
@@ -587,13 +654,18 @@ clutter_backend_set_double_click_distance (ClutterBackend *backend,
  * Return value: a distance, in pixels.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:double-click-distance instead
  */
 guint
 clutter_backend_get_double_click_distance (ClutterBackend *backend)
 {
-  g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
+  ClutterSettings *settings = clutter_settings_get_default ();
+  gint retval;
 
-  return backend->priv->double_click_distance;
+  g_object_get (settings, "double-click-distance", &retval, NULL);
+
+  return retval;
 }
 
 /**
@@ -610,35 +682,42 @@ clutter_backend_get_double_click_distance (ClutterBackend *backend)
  * Applications should never need to call this function.
  *
  * Since: 0.4
+ *
+ * Deprecated: Use #ClutterSettings:font-dpi instead
  */
 void
 clutter_backend_set_resolution (ClutterBackend *backend,
                                 gdouble         dpi)
 {
-  ClutterBackendPrivate *priv;
+  ClutterSettings *settings;
+  gint resolution;
 
   g_return_if_fail (CLUTTER_IS_BACKEND (backend));
 
-  priv = backend->priv;
-
   if (dpi < 0)
-    dpi = -1.0;
+    resolution = -1;
+  else
+    resolution = dpi * 1024;
 
-  priv->resolution = dpi;
-
-  if (CLUTTER_CONTEXT ()->font_map)
-    cogl_pango_font_map_set_resolution (CLUTTER_CONTEXT ()->font_map, dpi);
-
-  g_signal_emit (backend, backend_signals[RESOLUTION_CHANGED], 0);
+  settings = clutter_settings_get_default ();
+  g_object_set (settings, "font-dpi", resolution, NULL);
 }
 
 /**
  * clutter_backend_get_resolution:
  * @backend: a #ClutterBackend
  *
- * Gets the resolution for font handling on the screen; see
- * clutter_backend_set_resolution() for full details.
- * 
+ * Gets the resolution for font handling on the screen.
+ *
+ * The resolution is a scale factor between points specified in a
+ * #PangoFontDescription and cairo units. The default value is 96.0,
+ * meaning that a 10 point font will be 13 units
+ * high (10 * 96. / 72. = 13.3).
+ *
+ * Clutter will set the resolution using the current backend when
+ * initializing; the resolution is also stored in the
+ * #ClutterSettings:font-dpi property.
+ *
  * Return value: the current resolution, or -1 if no resolution
  *   has been set.
  *
@@ -647,9 +726,15 @@ clutter_backend_set_resolution (ClutterBackend *backend,
 gdouble
 clutter_backend_get_resolution (ClutterBackend *backend)
 {
+  ClutterSettings *settings;
+  gint resolution;
+
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), -1.0);
 
-  return backend->priv->resolution;
+  settings = clutter_settings_get_default ();
+  g_object_get (settings, "font-dpi", &resolution, NULL);
+
+  return resolution / 1024.0;
 }
 
 /**
@@ -670,7 +755,7 @@ clutter_backend_get_resolution (ClutterBackend *backend)
  * Since: 0.8
  */
 void
-clutter_backend_set_font_options (ClutterBackend       *backend,
+clutter_backend_set_font_options (ClutterBackend             *backend,
                                   const cairo_font_options_t *options)
 {
   ClutterBackendPrivate *priv;
@@ -742,25 +827,16 @@ clutter_backend_get_font_options (ClutterBackend *backend)
  * be parsed by the pango_font_description_from_string() function.
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:font-name instead
  */
 void
 clutter_backend_set_font_name (ClutterBackend *backend,
                                const gchar    *font_name)
 {
-  ClutterBackendPrivate *priv;
+  ClutterSettings *settings = clutter_settings_get_default ();
 
-  g_return_if_fail (CLUTTER_IS_BACKEND (backend));
-
-  priv = backend->priv;
-
-  g_free (priv->font_name);
-
-  if (font_name == NULL || *font_name == '\0')
-    priv->font_name = g_strdup (DEFAULT_FONT_NAME);
-  else
-    priv->font_name = g_strdup (font_name);
-
-  g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
+  g_object_set (settings, "font-name", font_name, NULL);
 }
 
 /**
@@ -774,25 +850,26 @@ clutter_backend_set_font_name (ClutterBackend *backend,
  *   owned by the #ClutterBackend and should never be modified or freed
  *
  * Since: 1.0
+ *
+ * Deprecated: 1.4: Use #ClutterSettings:font-name instead
  */
 G_CONST_RETURN gchar *
 clutter_backend_get_font_name (ClutterBackend *backend)
 {
   ClutterBackendPrivate *priv;
+  ClutterSettings *settings;
 
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), NULL);
 
   priv = backend->priv;
 
-  if (G_LIKELY (priv->font_name))
-    return priv->font_name;
+  settings = clutter_settings_get_default ();
 
-  /* if we have never been called then we need to set the
-   * default font and update everything that relies on the
-   * ::font-changed signal
+  /* XXX yuck. but we return a const pointer, so we need to
+   * store it in the backend
    */
-  priv->font_name = g_strdup (DEFAULT_FONT_NAME);
-  g_signal_emit (backend, backend_signals[FONT_CHANGED], 0);
+  g_free (priv->font_name);
+  g_object_get (settings, "font-name", &priv->font_name, NULL);
 
   return priv->font_name;
 }
