@@ -84,6 +84,8 @@
 #include "config.h"
 #endif
 
+#include <math.h>
+
 #include "clutter-bind-constraint.h"
 
 #include "clutter-constraint.h"
@@ -130,10 +132,8 @@ G_DEFINE_TYPE (ClutterBindConstraint,
                CLUTTER_TYPE_CONSTRAINT);
 
 static void
-source_allocation_changed (ClutterActor           *source,
-                           const ClutterActorBox  *allocation,
-                           ClutterAllocationFlags  flags,
-                           ClutterBindConstraint  *bind)
+source_queue_relayout (ClutterActor          *source,
+                       ClutterBindConstraint *bind)
 {
   if (bind->actor != NULL)
     clutter_actor_queue_relayout (bind->actor);
@@ -152,7 +152,6 @@ clutter_bind_constraint_update_allocation (ClutterConstraint *constraint,
                                            ClutterActorBox   *allocation)
 {
   ClutterBindConstraint *bind = CLUTTER_BIND_CONSTRAINT (constraint);
-  ClutterActorMeta *meta = CLUTTER_ACTOR_META (constraint);
   gfloat source_width, source_height;
   gfloat actor_width, actor_height;
   ClutterVertex source_position = { 0., };
@@ -160,9 +159,6 @@ clutter_bind_constraint_update_allocation (ClutterConstraint *constraint,
   if (bind->source == NULL)
     return;
 
-  if (!clutter_actor_meta_get_enabled (meta))
-    return;
-
   source_position.x = clutter_actor_get_x (bind->source);
   source_position.y = clutter_actor_get_y (bind->source);
   clutter_actor_get_size (bind->source, &source_width, &source_height);
@@ -209,6 +205,25 @@ clutter_bind_constraint_set_actor (ClutterActorMeta *meta,
   parent->set_actor (meta, new_actor);
 }
 
+static void
+clutter_bind_constraint_dispose (GObject *gobject)
+{
+  ClutterBindConstraint *bind = CLUTTER_BIND_CONSTRAINT (gobject);
+
+  if (bind->source != NULL)
+    {
+      g_signal_handlers_disconnect_by_func (bind->source,
+                                            G_CALLBACK (source_destroyed),
+                                            bind);
+      g_signal_handlers_disconnect_by_func (bind->source,
+                                            G_CALLBACK (source_queue_relayout),
+                                            bind);
+      bind->source = NULL;
+    }
+
+  G_OBJECT_CLASS (clutter_bind_constraint_parent_class)->dispose (gobject);
+}
+
 static void
 clutter_bind_constraint_set_property (GObject      *gobject,
                                       guint         prop_id,
@@ -271,10 +286,10 @@ clutter_bind_constraint_class_init (ClutterBindConstraintClass *klass)
   ClutterActorMetaClass *meta_class = CLUTTER_ACTOR_META_CLASS (klass);
   ClutterConstraintClass *constraint_class = CLUTTER_CONSTRAINT_CLASS (klass);
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
 
   gobject_class->set_property = clutter_bind_constraint_set_property;
   gobject_class->get_property = clutter_bind_constraint_get_property;
+  gobject_class->dispose = clutter_bind_constraint_dispose;
 
   meta_class->set_actor = clutter_bind_constraint_set_actor;
 
@@ -286,14 +301,12 @@ clutter_bind_constraint_class_init (ClutterBindConstraintClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_object ("source",
-                               P_("Source"),
-                               P_("The source of the binding"),
-                               CLUTTER_TYPE_ACTOR,
-                               CLUTTER_PARAM_READWRITE |
-                               G_PARAM_CONSTRUCT);
-  obj_props[PROP_SOURCE] = pspec;
-  g_object_class_install_property (gobject_class, PROP_SOURCE, pspec);
+  obj_props[PROP_SOURCE] =
+    g_param_spec_object ("source",
+                         P_("Source"),
+                         P_("The source of the binding"),
+                         CLUTTER_TYPE_ACTOR,
+                         CLUTTER_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
   /**
    * ClutterBindConstraint:coordinate:
@@ -302,15 +315,13 @@ clutter_bind_constraint_class_init (ClutterBindConstraintClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_enum ("coordinate",
-                             P_("Coordinate"),
-                             P_("The coordinate to bind"),
-                             CLUTTER_TYPE_BIND_COORDINATE,
-                             CLUTTER_BIND_X,
-                             CLUTTER_PARAM_READWRITE |
-                             G_PARAM_CONSTRUCT);
-  obj_props[PROP_COORDINATE] = pspec;
-  g_object_class_install_property (gobject_class, PROP_COORDINATE, pspec);
+  obj_props[PROP_COORDINATE] =
+    g_param_spec_enum ("coordinate",
+                       P_("Coordinate"),
+                       P_("The coordinate to bind"),
+                       CLUTTER_TYPE_BIND_COORDINATE,
+                       CLUTTER_BIND_X,
+                       CLUTTER_PARAM_READWRITE | G_PARAM_CONSTRUCT);
 
   /**
    * ClutterBindConstraint:offset:
@@ -319,15 +330,17 @@ clutter_bind_constraint_class_init (ClutterBindConstraintClass *klass)
    *
    * Since: 1.4
    */
-  pspec = g_param_spec_float ("offset",
-                              P_("Offset"),
-                              P_("The offset in pixels to apply to the binding"),
-                              -G_MAXFLOAT, G_MAXFLOAT,
-                              0.0f,
-                              CLUTTER_PARAM_READWRITE |
-                              G_PARAM_CONSTRUCT);
-  obj_props[PROP_OFFSET] = pspec;
-  g_object_class_install_property (gobject_class, PROP_OFFSET, pspec);
+  obj_props[PROP_OFFSET] =
+    g_param_spec_float ("offset",
+                        P_("Offset"),
+                        P_("The offset in pixels to apply to the binding"),
+                        -G_MAXFLOAT, G_MAXFLOAT,
+                        0.0f,
+                        CLUTTER_PARAM_READWRITE | G_PARAM_CONSTRUCT);
+
+  _clutter_object_class_install_properties (gobject_class,
+                                            PROP_LAST,
+                                            obj_props);
 }
 
 static void
@@ -395,15 +408,15 @@ clutter_bind_constraint_set_source (ClutterBindConstraint *constraint,
                                             G_CALLBACK (source_destroyed),
                                             constraint);
       g_signal_handlers_disconnect_by_func (old_source,
-                                            G_CALLBACK (source_allocation_changed),
+                                            G_CALLBACK (source_queue_relayout),
                                             constraint);
     }
 
   constraint->source = source;
   if (constraint->source != NULL)
     {
-      g_signal_connect (constraint->source, "allocation-changed",
-                        G_CALLBACK (source_allocation_changed),
+      g_signal_connect (constraint->source, "queue-relayout",
+                        G_CALLBACK (source_queue_relayout),
                         constraint);
       g_signal_connect (constraint->source, "destroy",
                         G_CALLBACK (source_destroyed),
