@@ -27,11 +27,12 @@
 #include "config.h"
 #endif
 
+#include "clutter-backend-private.h"
+#include "clutter-debug.h"
+#include "clutter-event.h"
 #include "clutter-keysyms.h"
 #include "clutter-keysyms-table.h"
-#include "clutter-event.h"
 #include "clutter-private.h"
-#include "clutter-debug.h"
 
 /**
  * SECTION:clutter-event
@@ -51,6 +52,10 @@ typedef struct _ClutterEventPrivate {
 
 static GHashTable *all_events = NULL;
 
+G_DEFINE_BOXED_TYPE (ClutterEvent, clutter_event,
+                     clutter_event_copy,
+                     clutter_event_free);
+
 static gboolean
 is_event_allocated (const ClutterEvent *event)
 {
@@ -98,7 +103,7 @@ _clutter_event_set_platform_data (ClutterEvent *event,
  * Return value: a #ClutterEventType
  */
 ClutterEventType
-clutter_event_type (ClutterEvent *event)
+clutter_event_type (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_NOTHING);
 
@@ -116,7 +121,7 @@ clutter_event_type (ClutterEvent *event)
  * Since: 0.4
  */
 guint32
-clutter_event_get_time (ClutterEvent *event)
+clutter_event_get_time (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_CURRENT_TIME);
 
@@ -134,7 +139,7 @@ clutter_event_get_time (ClutterEvent *event)
  * Since: 0.4
  */
 ClutterModifierType
-clutter_event_get_state (ClutterEvent *event)
+clutter_event_get_state (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
 
@@ -164,17 +169,17 @@ clutter_event_get_state (ClutterEvent *event)
 /**
  * clutter_event_get_coords:
  * @event: a #ClutterEvent
- * @x: return location for the X coordinate
- * @y: return location for the Y coordinate
+ * @x: (out): return location for the X coordinate, or %NULL
+ * @y: (out): return location for the Y coordinate, or %NULL
  *
  * Retrieves the coordinates of @event and puts them into @x and @y.
  *
  * Since: 0.4
  */
 void
-clutter_event_get_coords (ClutterEvent *event,
-                          gfloat       *x,
-                          gfloat       *y)
+clutter_event_get_coords (const ClutterEvent *event,
+                          gfloat             *x,
+                          gfloat             *y)
 {
   gfloat event_x, event_y;
 
@@ -235,7 +240,7 @@ clutter_event_get_coords (ClutterEvent *event,
  * Since: 0.6
  */
 ClutterActor *
-clutter_event_get_source (ClutterEvent *event)
+clutter_event_get_source (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
 
@@ -254,7 +259,7 @@ clutter_event_get_source (ClutterEvent *event)
  * Since: 0.8
  */
 ClutterStage *
-clutter_event_get_stage (ClutterEvent *event)
+clutter_event_get_stage (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
 
@@ -272,7 +277,7 @@ clutter_event_get_stage (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterEventFlags
-clutter_event_get_flags (ClutterEvent *event)
+clutter_event_get_flags (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_EVENT_NONE);
 
@@ -291,7 +296,7 @@ clutter_event_get_flags (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterActor *
-clutter_event_get_related (ClutterEvent *event)
+clutter_event_get_related (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, NULL);
   g_return_val_if_fail (event->type == CLUTTER_ENTER ||
@@ -311,7 +316,7 @@ clutter_event_get_related (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterScrollDirection
-clutter_event_get_scroll_direction (ClutterEvent *event)
+clutter_event_get_scroll_direction (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, CLUTTER_SCROLL_UP);
   g_return_val_if_fail (event->type == CLUTTER_SCROLL, CLUTTER_SCROLL_UP);
@@ -331,7 +336,7 @@ clutter_event_get_scroll_direction (ClutterEvent *event)
  * Since: 1.0
  */
 guint32
-clutter_event_get_button (ClutterEvent *event)
+clutter_event_get_button (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_BUTTON_PRESS ||
@@ -352,7 +357,7 @@ clutter_event_get_button (ClutterEvent *event)
  * Since: 1.0
  */
 guint32
-clutter_event_get_click_count (ClutterEvent *event)
+clutter_event_get_click_count (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_BUTTON_PRESS ||
@@ -375,7 +380,7 @@ clutter_event_get_click_count (ClutterEvent *event)
  * Since: 1.0
  */
 guint
-clutter_event_get_key_symbol (ClutterEvent *event)
+clutter_event_get_key_symbol (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
@@ -396,7 +401,7 @@ clutter_event_get_key_symbol (ClutterEvent *event)
  * Since: 1.0
  */
 guint16
-clutter_event_get_key_code (ClutterEvent *event)
+clutter_event_get_key_code (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
@@ -414,7 +419,7 @@ clutter_event_get_key_code (ClutterEvent *event)
  * Return value: The unicode value representing the key
  */
 guint32
-clutter_event_get_key_unicode (ClutterEvent *event)
+clutter_event_get_key_unicode (const ClutterEvent *event)
 {
   g_return_val_if_fail (event != NULL, 0);
   g_return_val_if_fail (event->type == CLUTTER_KEY_PRESS ||
@@ -483,7 +488,7 @@ clutter_keysym_to_unicode (guint keyval)
  *   no specific device set.
  */
 gint
-clutter_event_get_device_id (ClutterEvent *event)
+clutter_event_get_device_id (const ClutterEvent *event)
 {
   ClutterInputDevice *device = NULL;
 
@@ -516,7 +521,7 @@ clutter_event_get_device_id (ClutterEvent *event)
     }
 
   if (device != NULL)
-    return device->id;
+    return clutter_input_device_get_device_id (device);
   else
     return -1;
 }
@@ -533,7 +538,7 @@ clutter_event_get_device_id (ClutterEvent *event)
  * Since: 1.0
  */
 ClutterInputDeviceType
-clutter_event_get_device_type (ClutterEvent *event)
+clutter_event_get_device_type (const ClutterEvent *event)
 {
   ClutterInputDevice *device = NULL;
 
@@ -570,7 +575,7 @@ clutter_event_get_device_type (ClutterEvent *event)
     }
 
   if (device != NULL)
-    return device->device_type;
+    return clutter_input_device_get_device_type (device);
   else
     return CLUTTER_POINTER_DEVICE;
 }
@@ -584,12 +589,14 @@ clutter_event_get_device_type (ClutterEvent *event)
  * The #ClutterInputDevice structure is completely opaque and should
  * be cast to the platform-specific implementation.
  *
- * Return value: the #ClutterInputDevice or %NULL
+ * Return value: (transfer none): the #ClutterInputDevice or %NULL. The
+ *   returned device is owned by the #ClutterEvent and it should not
+ *   be unreferenced
  *
  * Since: 1.0
  */
 ClutterInputDevice *
-clutter_event_get_device (ClutterEvent *event)
+clutter_event_get_device (const ClutterEvent *event)
 {
   ClutterInputDevice *device = NULL;
 
@@ -628,18 +635,6 @@ clutter_event_get_device (ClutterEvent *event)
   return device;
 }
 
-GType
-clutter_event_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (!our_type)
-    our_type = g_boxed_type_register_static (I_("ClutterEvent"),
-		    			     (GBoxedCopyFunc) clutter_event_copy,
-					     (GBoxedFreeFunc) clutter_event_free);
-  return our_type;
-}
-
 /**
  * clutter_event_new:
  * @type: The type of event.
@@ -659,7 +654,7 @@ clutter_event_new (ClutterEventType type)
   new_event = (ClutterEvent *) priv;
   new_event->type = new_event->any.type = type;
 
-  if (all_events == NULL)
+  if (G_UNLIKELY (all_events == NULL))
     all_events = g_hash_table_new (NULL, NULL);
 
   g_hash_table_replace (all_events, priv, GUINT_TO_POINTER (1));
@@ -676,7 +671,7 @@ clutter_event_new (ClutterEventType type)
  * Return value: A newly allocated #ClutterEvent
  */
 ClutterEvent *
-clutter_event_copy (ClutterEvent *event)
+clutter_event_copy (const ClutterEvent *event)
 {
   ClutterEvent *new_event;
 
@@ -774,7 +769,7 @@ clutter_event_peek (void)
  * Since: 0.6
  */
 void
-clutter_event_put (ClutterEvent *event)
+clutter_event_put (const ClutterEvent *event)
 {
   ClutterMainContext *context = _clutter_context_get_default ();
   ClutterEvent       *event_copy;
