@@ -55,22 +55,24 @@
 
 #include "clutter-stage.h"
 
-#include "clutter-backend.h"
-#include "clutter-main.h"
+#include "clutter-actor-private.h"
+#include "clutter-backend-private.h"
 #include "clutter-color.h"
-#include "clutter-util.h"
+#include "clutter-container.h"
+#include "clutter-debug.h"
+#include "clutter-device-manager-private.h"
+#include "clutter-enum-types.h"
+#include "clutter-id-pool.h"
+#include "clutter-main.h"
 #include "clutter-marshal.h"
 #include "clutter-master-clock.h"
-#include "clutter-enum-types.h"
+#include "clutter-paint-volume-private.h"
 #include "clutter-private.h"
-#include "clutter-debug.h"
-#include "clutter-stage-manager.h"
-#include "clutter-stage-window.h"
-#include "clutter-version.h" 	/* For flavour */
-#include "clutter-id-pool.h"
-#include "clutter-container.h"
 #include "clutter-profile.h"
-#include "clutter-input-device.h"
+#include "clutter-stage-manager-private.h"
+#include "clutter-stage-private.h"
+#include "clutter-util.h"
+#include "clutter-version.h" 	/* For flavour */
 
 #include "cogl/cogl.h"
 
@@ -131,6 +133,8 @@ struct _ClutterStagePrivate
 
   GList              *pending_queue_redraws;
 
+  ClutterPickMode     pick_buffer_mode;
+
   guint relayout_pending       : 1;
   guint redraw_pending         : 1;
   guint is_fullscreen          : 1;
@@ -143,6 +147,7 @@ struct _ClutterStagePrivate
   guint dirty_viewport         : 1;
   guint dirty_projection       : 1;
   guint have_valid_pick_buffer : 1;
+  guint accept_focus           : 1;
 };
 
 enum
@@ -160,7 +165,8 @@ enum
   PROP_FOG,
   PROP_USE_ALPHA,
   PROP_KEY_FOCUS,
-  PROP_NO_CLEAR_HINT
+  PROP_NO_CLEAR_HINT,
+  PROP_ACCEPT_FOCUS
 };
 
 enum
@@ -268,10 +274,6 @@ clutter_stage_allocate (ClutterActor           *self,
   height = clutter_actor_box_get_height (box);
   _clutter_stage_window_get_geometry (priv->impl, &geom);
 
-  /* XXX: Until Cogl becomes fully responsible for backend windows Clutter
-   * need to manually keep it informed of the current window size */
-  _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
-
   /* if the stage is fixed size (for instance, it's using a frame-buffer)
    * then we simply ignore any allocation request and override the
    * allocation chain.
@@ -343,6 +345,14 @@ clutter_stage_allocate (ClutterActor           *self,
       klass->allocate (self, &override, flags);
     }
 
+  /* XXX: Until Cogl becomes fully responsible for backend windows
+   * Clutter need to manually keep it informed of the current window
+   * size. We do this after the allocation above so that the stage
+   * window has a chance to update the window size based on the
+   * allocation. */
+  _clutter_stage_window_get_geometry (priv->impl, &geom);
+  _cogl_onscreen_clutter_backend_set_size (geom.width, geom.height);
+
   clutter_actor_get_allocation_geometry (self, &geom);
   if (geom.width != prev_geom.width || geom.height != prev_geom.height)
     {
@@ -447,6 +457,15 @@ clutter_stage_pick (ClutterActor       *self,
                              NULL);
 }
 
+static gboolean
+clutter_stage_get_paint_volume (ClutterActor *self,
+                                ClutterPaintVolume *volume)
+{
+  /* Returning False effectively means Clutter has to assume it covers
+   * everything... */
+  return FALSE;
+}
+
 static void
 clutter_stage_realize (ClutterActor *self)
 {
@@ -744,11 +763,12 @@ _clutter_stage_maybe_relayout (ClutterActor *actor)
 
   if (!priv->relayout_pending)
     return;
-  priv->relayout_pending = FALSE;
 
   /* avoid reentrancy */
   if (!CLUTTER_ACTOR_IN_RELAYOUT (stage))
     {
+      priv->relayout_pending = FALSE;
+
       CLUTTER_TIMER_START (_clutter_uprof_context, relayout_timer);
       CLUTTER_NOTE (ACTOR, "Recomputing layout");
 
@@ -859,7 +879,10 @@ clutter_stage_real_queue_redraw (ClutterActor *actor,
     return;
 
   if (_clutter_stage_window_ignoring_redraw_clips (stage_window))
-    return;
+    {
+      _clutter_stage_window_add_redraw_clip (stage_window, NULL);
+      return;
+    }
 
   /* Convert the clip volume (which is in leaf actor coordinates) into stage
    * coordinates and then into an axis aligned stage coordinates bounding
@@ -1203,6 +1226,7 @@ clutter_stage_class_init (ClutterStageClass *klass)
   actor_class->get_preferred_height = clutter_stage_get_preferred_height;
   actor_class->paint = clutter_stage_paint;
   actor_class->pick = clutter_stage_pick;
+  actor_class->get_paint_volume = clutter_stage_get_paint_volume;
   actor_class->realize = clutter_stage_realize;
   actor_class->unrealize = clutter_stage_unrealize;
   actor_class->show = clutter_stage_show;
@@ -1402,6 +1426,20 @@ clutter_stage_class_init (ClutterStageClass *klass)
                                 CLUTTER_PARAM_READWRITE);
   g_object_class_install_property (gobject_class, PROP_NO_CLEAR_HINT, pspec);
 
+  /**
+   * ClutterStage:accept-focus:
+   *
+   * Whether the #ClutterStage should accept key focus when shown.
+   *
+   * Since: 1.6
+   */
+  pspec = g_param_spec_boolean ("accept-focus",
+                                P_("Accept Focus"),
+                                P_("Whether the stage should accept focus on show"),
+                                TRUE,
+                                CLUTTER_PARAM_READWRITE);
+  g_object_class_install_property (gobject_class, PROP_ACCEPT_FOCUS, pspec);
+
   /**
    * ClutterStage::fullscreen
    * @stage: the stage which was fullscreened
@@ -1581,7 +1619,7 @@ clutter_stage_init (ClutterStage *self)
   _clutter_stage_window_get_geometry (priv->impl, &geom);
   _clutter_stage_set_viewport (self, 0, 0, geom.width, geom.height);
 
-  _clutter_stage_set_pick_buffer_valid (self, FALSE);
+  _clutter_stage_set_pick_buffer_valid (self, FALSE, CLUTTER_PICK_ALL);
   _clutter_stage_reset_picks_per_frame_counter (self);
 
   priv->paint_volume_stack =
@@ -2388,7 +2426,7 @@ clutter_stage_get_key_focus (ClutterStage *stage)
  *
  * Gets whether the depth cueing effect is enabled on @stage.
  *
- * Return value: %TRUE if the the depth cueing effect is enabled
+ * Return value: %TRUE if the depth cueing effect is enabled
  *
  * Since: 0.6
  */
@@ -2549,17 +2587,9 @@ clutter_perspective_free (gpointer data)
     g_slice_free (ClutterPerspective, data);
 }
 
-GType
-clutter_perspective_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (!our_type)
-    our_type = g_boxed_type_register_static (I_("ClutterPerspective"),
-                                             clutter_perspective_copy,
-                                             clutter_perspective_free);
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (ClutterPerspective, clutter_perspective,
+                     clutter_perspective_copy,
+                     clutter_perspective_free);
 
 static gpointer
 clutter_fog_copy (gpointer data)
@@ -2577,18 +2607,7 @@ clutter_fog_free (gpointer data)
     g_slice_free (ClutterFog, data);
 }
 
-GType
-clutter_fog_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (G_UNLIKELY (our_type == 0))
-    our_type = g_boxed_type_register_static (I_("ClutterFog"),
-                                             clutter_fog_copy,
-                                             clutter_fog_free);
-
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (ClutterFog, clutter_fog, clutter_fog_copy, clutter_fog_free);
 
 /**
  * clutter_stage_new:
@@ -2597,7 +2616,7 @@ clutter_fog_get_type (void)
  * top-level actor which can be used as another container. It works
  * exactly like the default stage, but while clutter_stage_get_default()
  * will always return the same instance, you will have to keep a pointer
- * to any #ClutterStage returned by clutter_stage_create().
+ * to any #ClutterStage returned by clutter_stage_new().
  *
  * The ability to support multiple stages depends on the current
  * backend. Use clutter_feature_available() and
@@ -2688,10 +2707,16 @@ _clutter_stage_maybe_setup_viewport (ClutterStage *stage)
                          priv->viewport[2],
                          priv->viewport[3]);
 
+
+      priv->dirty_viewport = FALSE;
     }
 
   if (priv->dirty_projection)
-    cogl_set_projection_matrix (&priv->projection);
+    {
+      cogl_set_projection_matrix (&priv->projection);
+
+      priv->dirty_projection = FALSE;
+    }
 }
 
 /**
@@ -3066,20 +3091,25 @@ clutter_stage_get_no_clear_hint (ClutterStage *stage)
 }
 
 gboolean
-_clutter_stage_get_pick_buffer_valid (ClutterStage *stage)
+_clutter_stage_get_pick_buffer_valid (ClutterStage *stage, ClutterPickMode mode)
 {
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
 
+  if (stage->priv->pick_buffer_mode != mode)
+    return FALSE;
+
   return stage->priv->have_valid_pick_buffer;
 }
 
 void
-_clutter_stage_set_pick_buffer_valid (ClutterStage *stage,
-                                      gboolean      valid)
+_clutter_stage_set_pick_buffer_valid (ClutterStage   *stage,
+                                      gboolean        valid,
+                                      ClutterPickMode mode)
 {
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
 
   stage->priv->have_valid_pick_buffer = !!valid;
+  stage->priv->pick_buffer_mode = mode;
 }
 
 void
@@ -3185,7 +3215,7 @@ _clutter_stage_queue_actor_redraw (ClutterStage *stage,
    * state changes that affects painting *or* picking so we can use
    * this point to invalidate any currently cached pick buffer.
    */
-  _clutter_stage_set_pick_buffer_valid (stage, FALSE);
+  _clutter_stage_set_pick_buffer_valid (stage, FALSE, -1);
 
   if (entry)
     {
@@ -3199,6 +3229,11 @@ _clutter_stage_queue_actor_redraw (ClutterStage *stage,
        * clip together with the existing clip */
       if (clip)
         clutter_paint_volume_union (&entry->clip, clip);
+      else
+        {
+          clutter_paint_volume_free (&entry->clip);
+          entry->has_clip = FALSE;
+        }
       return entry;
     }
   else
@@ -3225,18 +3260,25 @@ _clutter_stage_queue_actor_redraw (ClutterStage *stage,
 static void
 free_queue_redraw_entry (ClutterStageQueueRedrawEntry *entry)
 {
-  g_object_unref (entry->actor);
+  if (entry->actor)
+    g_object_unref (entry->actor);
   if (entry->has_clip)
     clutter_paint_volume_free (&entry->clip);
   g_slice_free (ClutterStageQueueRedrawEntry, entry);
 }
 
 void
-_clutter_stage_queue_redraw_entry_invalidate (
-                                          ClutterStageQueueRedrawEntry *entry)
+_clutter_stage_queue_redraw_entry_invalidate (ClutterStageQueueRedrawEntry *entry)
 {
-  g_object_unref (entry->actor);
-  entry->actor = NULL;
+  if (entry == NULL)
+    return;
+
+  if (entry->actor != NULL)
+    {
+      g_object_unref (entry->actor);
+      entry->actor = NULL;
+    }
+
   if (entry->has_clip)
     {
       clutter_paint_volume_free (&entry->clip);
@@ -3266,17 +3308,68 @@ _clutter_stage_maybe_finish_queue_redraws (ClutterStage *stage)
       for (l = stolen_list; l; l = l->next)
         {
           ClutterStageQueueRedrawEntry *entry = l->data;
+          ClutterPaintVolume *clip;
 
           /* NB: Entries may be invalidated if the actor gets destroyed */
-          if (G_UNLIKELY (entry->actor == NULL))
-            continue;
+          if (G_LIKELY (entry->actor != NULL))
+	    {
+	      clip = entry->has_clip ? &entry->clip : NULL;
 
-          ClutterPaintVolume *clip = entry->has_clip ? &entry->clip : NULL;
+	      _clutter_actor_finish_queue_redraw (entry->actor, clip);
+	    }
 
-          _clutter_actor_finish_queue_redraw (entry->actor, clip);
           free_queue_redraw_entry (entry);
         }
       g_list_free (stolen_list);
     }
 }
 
+/**
+ * clutter_stage_get_accept_focus:
+ * @stage: a #ClutterStage
+ * @accept_focus: %TRUE to accept focus on show
+ *
+ * Sets whether the @stage should accept the key focus when shown.
+ *
+ * This function should be called before showing @stage using
+ * clutter_actor_show().
+ *
+ * Since: 1.6
+ */
+void
+clutter_stage_set_accept_focus (ClutterStage *stage,
+                                gboolean      accept_focus)
+{
+  ClutterStagePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_STAGE (stage));
+
+  accept_focus = !!accept_focus;
+
+  priv = stage->priv;
+
+  if (priv->accept_focus != accept_focus)
+    {
+      _clutter_stage_window_set_accept_focus (priv->impl, accept_focus);
+      g_object_notify (G_OBJECT (stage), "accept-focus");
+    }
+}
+
+/**
+ * clutter_stage_get_accept_focus:
+ * @stage: a #ClutterStage
+ *
+ * Retrieves the value set with clutter_stage_set_accept_focus().
+ *
+ * Return value: %TRUE if the #ClutterStage should accept focus, and %FALSE
+ *   otherwise
+ *
+ * Since: 1.6
+ */
+gboolean
+clutter_stage_get_accept_focus (ClutterStage *stage)
+{
+  g_return_val_if_fail (CLUTTER_IS_STAGE (stage), TRUE);
+
+  return stage->priv->accept_focus;
+}
