@@ -64,12 +64,16 @@ struct _ClutterTimelinePrivate
   GHashTable *markers_by_name;
 
   /* Time we last advanced the elapsed time and showed a frame */
-  GTimeVal last_frame_time;
+  gint64 last_frame_time;
 
-  guint loop       : 1;
-  guint is_playing : 1;
-  /* If we've just started playing and haven't yet gotten a tick from the master clock */
+  guint loop               : 1;
+  guint is_playing         : 1;
+
+  /* If we've just started playing and haven't yet gotten
+   * a tick from the master clock
+   */
   guint waiting_first_tick : 1;
+  guint auto_reverse       : 1;
 };
 
 typedef struct {
@@ -86,11 +90,12 @@ enum
   PROP_DELAY,
   PROP_DURATION,
   PROP_DIRECTION,
+  PROP_AUTO_REVERSE,
 
   PROP_LAST
 };
 
-static GParamSpec *obj_props[PROP_LAST];
+static GParamSpec *obj_props[PROP_LAST] = { NULL, };
 
 enum
 {
@@ -138,20 +143,16 @@ clutter_timeline_set_property (GObject      *object,
 			       const GValue *value,
 			       GParamSpec   *pspec)
 {
-  ClutterTimeline        *timeline;
-  ClutterTimelinePrivate *priv;
-
-  timeline = CLUTTER_TIMELINE(object);
-  priv = timeline->priv;
+  ClutterTimeline *timeline = CLUTTER_TIMELINE (object);
 
   switch (prop_id)
     {
     case PROP_LOOP:
-      priv->loop = g_value_get_boolean (value);
+      clutter_timeline_set_loop (timeline, g_value_get_boolean (value));
       break;
 
     case PROP_DELAY:
-      priv->delay = g_value_get_uint (value);
+      clutter_timeline_set_delay (timeline, g_value_get_uint (value));
       break;
 
     case PROP_DURATION:
@@ -162,6 +163,10 @@ clutter_timeline_set_property (GObject      *object,
       clutter_timeline_set_direction (timeline, g_value_get_enum (value));
       break;
 
+    case PROP_AUTO_REVERSE:
+      clutter_timeline_set_auto_reverse (timeline, g_value_get_boolean (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -174,11 +179,8 @@ clutter_timeline_get_property (GObject    *object,
 			       GValue     *value,
 			       GParamSpec *pspec)
 {
-  ClutterTimeline        *timeline;
-  ClutterTimelinePrivate *priv;
-
-  timeline = CLUTTER_TIMELINE(object);
-  priv = timeline->priv;
+  ClutterTimeline *timeline = CLUTTER_TIMELINE (object);
+  ClutterTimelinePrivate *priv = timeline->priv;
 
   switch (prop_id)
     {
@@ -198,6 +200,10 @@ clutter_timeline_get_property (GObject    *object,
       g_value_set_enum (value, priv->direction);
       break;
 
+    case PROP_AUTO_REVERSE:
+      g_value_set_boolean (value, priv->auto_reverse);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -244,12 +250,6 @@ static void
 clutter_timeline_class_init (ClutterTimelineClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GParamSpec *pspec;
-
-  object_class->set_property = clutter_timeline_set_property;
-  object_class->get_property = clutter_timeline_get_property;
-  object_class->finalize     = clutter_timeline_finalize;
-  object_class->dispose      = clutter_timeline_dispose;
 
   g_type_class_add_private (klass, sizeof (ClutterTimelinePrivate));
 
@@ -258,13 +258,12 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * Whether the timeline should automatically rewind and restart.
    */
-  pspec = g_param_spec_boolean ("loop",
-                                "Loop",
-                                "Should the timeline automatically restart",
-                                FALSE,
-                                CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_LOOP] = pspec;
-  g_object_class_install_property (object_class, PROP_LOOP, pspec);
+  obj_props[PROP_LOOP] =
+    g_param_spec_boolean ("loop",
+                          P_("Loop"),
+                          P_("Should the timeline automatically restart"),
+                          FALSE,
+                          CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterTimeline:delay:
@@ -274,14 +273,13 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * Since: 0.4
    */
-  pspec = g_param_spec_uint ("delay",
-                             "Delay",
-                             "Delay before start",
-                             0, G_MAXUINT,
-                             0,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DELAY] = pspec;
-  g_object_class_install_property (object_class, PROP_DELAY, pspec);
+  obj_props[PROP_DELAY] =
+    g_param_spec_uint ("delay",
+                       P_("Delay"),
+                       P_("Delay before start"),
+                       0, G_MAXUINT,
+                       0,
+                       CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterTimeline:duration:
@@ -291,14 +289,13 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * Since: 0.6
    */
-  pspec = g_param_spec_uint ("duration",
-                             "Duration",
-                             "Duration of the timeline in milliseconds",
-                             0, G_MAXUINT,
-                             1000,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DURATION] = pspec;
-  g_object_class_install_property (object_class, PROP_DURATION, pspec);
+  obj_props[PROP_DURATION] =
+    g_param_spec_uint ("duration",
+                       P_("Duration"),
+                       P_("Duration of the timeline in milliseconds"),
+                       0, G_MAXUINT,
+                       1000,
+                       CLUTTER_PARAM_READWRITE);
 
   /**
    * ClutterTimeline:direction:
@@ -308,14 +305,36 @@ clutter_timeline_class_init (ClutterTimelineClass *klass)
    *
    * Since: 0.6
    */
-  pspec = g_param_spec_enum ("direction",
-                             "Direction",
-                             "Direction of the timeline",
-                             CLUTTER_TYPE_TIMELINE_DIRECTION,
-                             CLUTTER_TIMELINE_FORWARD,
-                             CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_DIRECTION] = pspec;
-  g_object_class_install_property (object_class, PROP_DIRECTION, pspec);
+  obj_props[PROP_DIRECTION] =
+    g_param_spec_enum ("direction",
+                       P_("Direction"),
+                       P_("Direction of the timeline"),
+                       CLUTTER_TYPE_TIMELINE_DIRECTION,
+                       CLUTTER_TIMELINE_FORWARD,
+                       CLUTTER_PARAM_READWRITE);
+
+  /**
+   * ClutterTimeline:auto-reverse:
+   *
+   * If the direction of the timeline should be automatically reversed
+   * when reaching the end.
+   *
+   * Since: 1.6
+   */
+  obj_props[PROP_AUTO_REVERSE] =
+    g_param_spec_boolean ("auto-reverse",
+                          P_("Auto Reverse"),
+                          P_("Whether the direction should be reversed when reaching the end"),
+                          FALSE,
+                          CLUTTER_PARAM_READWRITE);
+
+  object_class->dispose      = clutter_timeline_dispose;
+  object_class->finalize     = clutter_timeline_finalize;
+  object_class->set_property = clutter_timeline_set_property;
+  object_class->get_property = clutter_timeline_get_property;
+  _clutter_object_class_install_properties (object_class,
+                                            PROP_LAST,
+                                            obj_props);
 
   /**
    * ClutterTimeline::new-frame:
@@ -664,6 +683,18 @@ clutter_timeline_do_frame (ClutterTimeline *timeline)
 
       g_signal_emit (timeline, timeline_signals[COMPLETED], 0);
 
+      if (priv->auto_reverse)
+        {
+          /* :auto-reverse changes the direction of the timeline */
+          if (priv->direction == CLUTTER_TIMELINE_FORWARD)
+            priv->direction = CLUTTER_TIMELINE_BACKWARD;
+          else
+            priv->direction = CLUTTER_TIMELINE_FORWARD;
+
+          _clutter_notify_by_pspec (G_OBJECT (timeline),
+                                    obj_props[PROP_DIRECTION]);
+        }
+
       /* Again check to see if the user has manually played with
        * the elapsed time, before we finally stop or loop the timeline */
 
@@ -694,9 +725,9 @@ clutter_timeline_do_frame (ClutterTimeline *timeline)
              time without emitting the new frame signal so we need to
              check for markers again */
           check_markers (timeline,
-                         priv->direction == CLUTTER_TIMELINE_FORWARD ?
-                         priv->elapsed_time :
-                         priv->duration - priv->elapsed_time);
+                         priv->direction == CLUTTER_TIMELINE_FORWARD
+                           ? priv->elapsed_time
+                           : priv->duration - priv->elapsed_time);
 
           g_object_unref (timeline);
           return TRUE;
@@ -1207,14 +1238,14 @@ clutter_timeline_get_delta (ClutterTimeline *timeline)
  * @timeline: a #ClutterTimeline
  * @tick_time: time of advance
  *
- * Advances @timeline based on the time passed in @msecs. This
+ * Advances @timeline based on the time passed in @tick_time. This
  * function is called by the master clock. The @timeline will use this
  * interval to emit the #ClutterTimeline::new-frame signal and
  * eventually skip frames.
  */
 void
-clutter_timeline_do_tick (ClutterTimeline *timeline,
-			  GTimeVal        *tick_time)
+_clutter_timeline_do_tick (ClutterTimeline *timeline,
+                           gint64           tick_time)
 {
   ClutterTimelinePrivate *priv;
 
@@ -1224,15 +1255,14 @@ clutter_timeline_do_tick (ClutterTimeline *timeline,
 
   if (priv->waiting_first_tick)
     {
-      priv->last_frame_time = *tick_time;
+      priv->last_frame_time = tick_time;
       priv->waiting_first_tick = FALSE;
     }
   else
     {
       gint64 msecs;
 
-      msecs = (tick_time->tv_sec - priv->last_frame_time.tv_sec) * 1000
-            + (tick_time->tv_usec - priv->last_frame_time.tv_usec) / 1000;
+      msecs = tick_time - priv->last_frame_time;
 
       /* if the clock rolled back between ticks we need to
        * account for it; the best course of action, since the
@@ -1241,14 +1271,14 @@ clutter_timeline_do_tick (ClutterTimeline *timeline,
        */
       if (msecs < 0)
         {
-          priv->last_frame_time = *tick_time;
+          priv->last_frame_time = tick_time;
           return;
         }
 
       if (msecs != 0)
 	{
 	  /* Avoid accumulating error */
-	  g_time_val_add (&priv->last_frame_time, msecs * 1000L);
+          priv->last_frame_time += msecs;
 	  priv->msecs_delta = msecs;
 	  clutter_timeline_do_frame (timeline);
 	}
@@ -1504,3 +1534,87 @@ clutter_timeline_has_marker (ClutterTimeline *timeline,
   return NULL != g_hash_table_lookup (timeline->priv->markers_by_name,
                                       marker_name);
 }
+
+/**
+ * clutter_timeline_set_auto_reverse:
+ * @timeline: a #ClutterTimeline
+ * @reverse: %TRUE if the @timeline should reverse the direction
+ *
+ * Sets whether @timeline should reverse the direction after the
+ * emission of the #ClutterTimeline::completed signal.
+ *
+ * Setting the #ClutterTimeline:reverse property to %TRUE is the
+ * equivalent of connecting a callback to the #ClutterTimeline::completed
+ * signal and changing the direction of the timeline from that callback;
+ * for instance, this code:
+ *
+ * |[
+ * static void
+ * reverse_timeline (ClutterTimeline *timeline)
+ * {
+ *   ClutterTimelineDirection dir = clutter_timeline_get_direction (timeline);
+ *
+ *   if (dir == CLUTTER_TIMELINE_FORWARD)
+ *     dir = CLUTTER_TIMELINE_BACKWARD;
+ *   else
+ *     dir = CLUTTER_TIMELINE_FORWARD;
+ *
+ *   clutter_timeline_set_direction (timeline, dir);
+ * }
+ * ...
+ *   timeline = clutter_timeline_new (1000);
+ *   clutter_timeline_set_loop (timeline);
+ *   g_signal_connect (timeline, "completed",
+ *                     G_CALLBACK (reverse_timeline),
+ *                     NULL);
+ * ]|
+ *
+ * can be effectively replaced by:
+ *
+ * |[
+ *   timeline = clutter_timeline_new (1000);
+ *   clutter_timeline_set_loop (timeline);
+ *   clutter_timeline_set_auto_reverse (timeline);
+ * ]|
+ *
+ * Since: 1.6
+ */
+void
+clutter_timeline_set_auto_reverse (ClutterTimeline *timeline,
+                                   gboolean         reverse)
+{
+  ClutterTimelinePrivate *priv;
+
+  g_return_if_fail (CLUTTER_IS_TIMELINE (timeline));
+
+  reverse = !!reverse;
+
+  priv = timeline->priv;
+
+  if (priv->auto_reverse != reverse)
+    {
+      priv->auto_reverse = reverse;
+
+      _clutter_notify_by_pspec (G_OBJECT (timeline),
+                                obj_props[PROP_AUTO_REVERSE]);
+    }
+}
+
+/**
+ * clutter_timeline_get_auto_reverse:
+ * @timeline: a #ClutterTimeline
+ *
+ * Retrieves the value set by clutter_timeline_set_auto_reverse().
+ *
+ * Return value: %TRUE if the timeline should automatically reverse, and
+ *   %FALSE otherwise
+ *
+ * Since: 1.6
+ */
+gboolean
+clutter_timeline_get_auto_reverse (ClutterTimeline *timeline)
+{
+  g_return_val_if_fail (CLUTTER_IS_TIMELINE (timeline), FALSE);
+
+  return timeline->priv->auto_reverse;
+}
