@@ -58,7 +58,9 @@ typedef enum {
   COGL_DEBUG_TEXTURE_PIXMAP   = 1 << 25,
   COGL_DEBUG_BITMAP           = 1 << 26,
   COGL_DEBUG_DISABLE_NPOT_TEXTURES = 1 << 27,
-  COGL_DEBUG_WIREFRAME        = 1 << 28
+  COGL_DEBUG_WIREFRAME        = 1 << 28,
+  COGL_DEBUG_DISABLE_SOFTWARE_CLIP = 1 << 29,
+  COGL_DEBUG_DISABLE_PROGRAM_CACHES = 1 << 30
 } CoglDebugFlags;
 
 #ifdef COGL_ENABLE_DEBUG
