@@ -37,11 +37,7 @@
 #include "cogl-clip-stack.h"
 #include "cogl-journal-private.h"
 
-#ifdef HAVE_COGL_GLES2
-
-#include "../gles/cogl-gles2-wrapper.h"
-
-#else
+#ifndef HAVE_COGL_GLES2
 
 #define glGenRenderbuffers                ctx->drv.pf_glGenRenderbuffers
 #define glDeleteRenderbuffers             ctx->drv.pf_glDeleteRenderbuffers
@@ -139,12 +135,14 @@ _cogl_is_framebuffer (void *object)
 static void
 _cogl_framebuffer_init (CoglFramebuffer *framebuffer,
                         CoglFramebufferType type,
+                        CoglPixelFormat format,
                         int width,
                         int height)
 {
   framebuffer->type             = type;
   framebuffer->width            = width;
   framebuffer->height           = height;
+  framebuffer->format           = format;
   framebuffer->viewport_x       = 0;
   framebuffer->viewport_y       = 0;
   framebuffer->viewport_width   = width;
@@ -164,10 +162,10 @@ _cogl_framebuffer_free (CoglFramebuffer *framebuffer)
 {
   _cogl_clip_state_destroy (&framebuffer->clip_state);
 
-  _cogl_matrix_stack_destroy (framebuffer->modelview_stack);
+  cogl_object_unref (framebuffer->modelview_stack);
   framebuffer->modelview_stack = NULL;
 
-  _cogl_matrix_stack_destroy (framebuffer->projection_stack);
+  cogl_object_unref (framebuffer->projection_stack);
   framebuffer->projection_stack = NULL;
 }
 
@@ -324,10 +322,18 @@ _cogl_framebuffer_init_bits (CoglFramebuffer *framebuffer)
   framebuffer->dirty_bitmasks = FALSE;
 }
 
+typedef struct
+{
+  CoglHandle texture;
+  unsigned int level;
+  unsigned int level_width;
+  unsigned int level_height;
+} CoglFramebufferTryFBOData;
+
 static gboolean
 try_creating_fbo (CoglOffscreen *offscreen,
                   TryFBOFlags flags,
-                  CoglHandle texture)
+                  CoglFramebufferTryFBOData *data)
 {
   GLuint gl_depth_stencil_handle;
   GLuint gl_depth_handle;
@@ -339,7 +345,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  if (!cogl_texture_get_gl_texture (texture, &tex_gl_handle, &tex_gl_target))
+  if (!cogl_texture_get_gl_texture (data->texture,
+                                    &tex_gl_handle, &tex_gl_target))
     return FALSE;
 
   if (tex_gl_target != GL_TEXTURE_2D
@@ -362,7 +369,7 @@ try_creating_fbo (CoglOffscreen *offscreen,
   offscreen->fbo_handle = fbo_gl_handle;
 
   GE (glFramebufferTexture2D (GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,
-			      tex_gl_target, tex_gl_handle, 0));
+                              tex_gl_target, tex_gl_handle, data->level));
 
   if (flags & _TRY_DEPTH_STENCIL)
     {
@@ -370,8 +377,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
       GE (glGenRenderbuffers (1, &gl_depth_stencil_handle));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, gl_depth_stencil_handle));
       GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_STENCIL,
-                                 cogl_texture_get_width (texture),
-                                 cogl_texture_get_height (texture)));
+                                 data->level_width,
+                                 data->level_height));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_STENCIL_ATTACHMENT,
@@ -391,8 +398,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
       /* For now we just ask for GL_DEPTH_COMPONENT16 since this is all that's
        * available under GLES */
       GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_DEPTH_COMPONENT16,
-                                 cogl_texture_get_width (texture),
-                                 cogl_texture_get_height (texture)));
+                                 data->level_width,
+                                 data->level_height));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_DEPTH_ATTACHMENT,
@@ -407,8 +414,8 @@ try_creating_fbo (CoglOffscreen *offscreen,
       GE (glGenRenderbuffers (1, &gl_stencil_handle));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, gl_stencil_handle));
       GE (glRenderbufferStorage (GL_RENDERBUFFER, GL_STENCIL_INDEX8,
-                                 cogl_texture_get_width (texture),
-                                 cogl_texture_get_height (texture)));
+                                 data->level_width,
+                                 data->level_height));
       GE (glBindRenderbuffer (GL_RENDERBUFFER, 0));
       GE (glFramebufferRenderbuffer (GL_FRAMEBUFFER,
                                      GL_STENCIL_ATTACHMENT,
@@ -443,11 +450,16 @@ try_creating_fbo (CoglOffscreen *offscreen,
 }
 
 CoglHandle
-cogl_offscreen_new_to_texture (CoglHandle texhandle)
+_cogl_offscreen_new_to_texture_full (CoglHandle texhandle,
+                                     CoglOffscreenFlags create_flags,
+                                     unsigned int level)
 {
   CoglOffscreen      *offscreen;
   static TryFBOFlags  flags;
   static gboolean     have_working_flags = FALSE;
+  unsigned int        i;
+  CoglFramebufferTryFBOData data;
+  gboolean            fbo_created;
 
   _COGL_GET_CONTEXT (ctx, COGL_INVALID_HANDLE);
 
@@ -462,6 +474,27 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
   if (cogl_texture_is_sliced (texhandle))
     return COGL_INVALID_HANDLE;
 
+  data.texture = texhandle;
+  data.level = level;
+
+  /* Calculate the size of the texture at this mipmap level to ensure
+     that it's a valid level */
+  data.level_width = cogl_texture_get_width (texhandle);
+  data.level_height = cogl_texture_get_height (texhandle);
+
+  for (i = 0; i < level; i++)
+    {
+      /* If neither dimension can be further divided then the level is
+         invalid */
+      if (data.level_width == 1 && data.level_height == 1)
+        return COGL_INVALID_HANDLE;
+
+      if (data.level_width > 1)
+        data.level_width >>= 1;
+      if (data.level_height > 1)
+        data.level_height >>= 1;
+    }
+
   /* XXX: The framebuffer_object spec isn't clear in defining whether attaching
    * a texture as a renderbuffer with mipmap filtering enabled while the
    * mipmaps have not been uploaded should result in an incomplete framebuffer
@@ -470,32 +503,44 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
    * To avoid an error with drivers that do consider this a problem we
    * explicitly set non mipmapped filters here. These will later be reset when
    * the texture is actually used for rendering according to the filters set on
-   * the corresponding CoglMaterial.
+   * the corresponding CoglPipeline.
    */
   _cogl_texture_set_filters (texhandle, GL_NEAREST, GL_NEAREST);
 
   offscreen = g_new0 (CoglOffscreen, 1);
   offscreen->texture = cogl_handle_ref (texhandle);
 
-  if ((have_working_flags &&
-       try_creating_fbo (offscreen, flags, texhandle)) ||
-#ifdef HAVE_COGL_GL
-      try_creating_fbo (offscreen, flags = _TRY_DEPTH_STENCIL, texhandle) ||
-#endif
-      try_creating_fbo (offscreen, flags = _TRY_DEPTH | _TRY_STENCIL,
-                        texhandle) ||
-      try_creating_fbo (offscreen, flags = _TRY_STENCIL, texhandle) ||
-      try_creating_fbo (offscreen, flags = _TRY_DEPTH, texhandle) ||
-      try_creating_fbo (offscreen, flags = 0, texhandle))
+  if ((create_flags & COGL_OFFSCREEN_DISABLE_DEPTH_AND_STENCIL))
+    fbo_created = try_creating_fbo (offscreen, 0, &data);
+  else
     {
-      /* Record that the last set of flags succeeded so that we can
-         try that set first next time */
-      have_working_flags = TRUE;
+      if ((have_working_flags &&
+           try_creating_fbo (offscreen, flags, &data)) ||
+#ifdef HAVE_COGL_GL
+          try_creating_fbo (offscreen, flags = _TRY_DEPTH_STENCIL, &data) ||
+#endif
+          try_creating_fbo (offscreen, flags = _TRY_DEPTH | _TRY_STENCIL,
+                            &data) ||
+          try_creating_fbo (offscreen, flags = _TRY_STENCIL, &data) ||
+          try_creating_fbo (offscreen, flags = _TRY_DEPTH, &data) ||
+          try_creating_fbo (offscreen, flags = 0, &data))
+        {
+          /* Record that the last set of flags succeeded so that we can
+             try that set first next time */
+          have_working_flags = TRUE;
+          fbo_created = TRUE;
+        }
+      else
+        fbo_created = FALSE;
+    }
 
+  if (fbo_created)
+    {
       _cogl_framebuffer_init (COGL_FRAMEBUFFER (offscreen),
                               COGL_FRAMEBUFFER_TYPE_OFFSCREEN,
-                              cogl_texture_get_width (texhandle),
-                              cogl_texture_get_height (texhandle));
+                              cogl_texture_get_format (texhandle),
+                              data.level_width,
+                              data.level_height);
 
       return _cogl_offscreen_object_new (offscreen);
     }
@@ -508,6 +553,12 @@ cogl_offscreen_new_to_texture (CoglHandle texhandle)
     }
 }
 
+CoglHandle
+cogl_offscreen_new_to_texture (CoglHandle texhandle)
+{
+  return _cogl_offscreen_new_to_texture_full (texhandle, 0, 0);
+}
+
 static void
 _cogl_offscreen_free (CoglOffscreen *offscreen)
 {
@@ -542,9 +593,21 @@ _cogl_onscreen_new (void)
    * implement CoglOnscreen framebuffers, since we can't, e.g. keep track of
    * the window size. */
 
+  /* FIXME: We are assuming onscreen buffers will always be
+     premultiplied so we'll set the premult flag on the bitmap
+     format. This will usually be correct because the result of the
+     default blending operations for Cogl ends up with premultiplied
+     data in the framebuffer. However it is possible for the
+     framebuffer to be in whatever format depending on what
+     CoglPipeline is used to render to it. Eventually we may want to
+     add a way for an application to inform Cogl that the framebuffer
+     is not premultiplied in case it is being used for some special
+     purpose. */
+
   onscreen = g_new0 (CoglOnscreen, 1);
   _cogl_framebuffer_init (COGL_FRAMEBUFFER (onscreen),
                           COGL_FRAMEBUFFER_TYPE_ONSCREEN,
+                          COGL_PIXEL_FORMAT_RGBA_8888_PRE,
                           0xdeadbeef, /* width */
                           0xdeadbeef); /* height */
 
@@ -636,7 +699,7 @@ _cogl_set_framebuffer_real (CoglFramebuffer *framebuffer)
    * we flush */
   _cogl_matrix_stack_dirty (framebuffer->modelview_stack);
   _cogl_matrix_stack_dirty (framebuffer->projection_stack);
-  _cogl_clip_state_dirty (&framebuffer->clip_state);
+  _cogl_clip_stack_dirty ();
 }
 
 void
@@ -783,7 +846,8 @@ _cogl_framebuffer_flush_state (CoglFramebuffer *framebuffer,
   /* XXX: Flushing clip state may trash the modelview and projection
    * matrices so we must do it before flushing the matrices...
    */
-  _cogl_clip_state_flush (&framebuffer->clip_state);
+  if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_CLIP_STATE))
+    _cogl_clip_state_flush (&framebuffer->clip_state);
 
   if (!(flags & COGL_FRAMEBUFFER_FLUSH_SKIP_MODELVIEW))
     _cogl_matrix_stack_flush_to_gl (framebuffer->modelview_stack,
