@@ -37,14 +37,9 @@
 #include "cogl-context.h"
 #include "cogl-texture-private.h"
 
-#ifdef COGL_PIPELINE_BACKEND_GLSL
-#include "cogl-pipeline-glsl-private.h"
-#endif
-#ifdef COGL_PIPELINE_BACKEND_ARBFP
-#include "cogl-pipeline-arbfp-private.h"
-#endif
-#ifdef COGL_PIPELINE_BACKEND_FIXED
-#include "cogl-pipeline-fixed-private.h"
+/* This is needed to set the color attribute on GLES2 */
+#ifdef HAVE_COGL_GLES2
+#include "cogl-pipeline-progend-glsl-private.h"
 #endif
 
 #include <glib.h>
@@ -54,10 +49,6 @@
  * GL/GLES compatability defines for pipeline thingies:
  */
 
-#ifdef HAVE_COGL_GLES2
-#include "../gles/cogl-gles2-wrapper.h"
-#endif
-
 #ifdef HAVE_COGL_GL
 #define glActiveTexture ctx->drv.pf_glActiveTexture
 #define glClientActiveTexture ctx->drv.pf_glClientActiveTexture
@@ -90,9 +81,9 @@ static void
 texture_unit_init (CoglTextureUnit *unit, int index_)
 {
   unit->index = index_;
-  unit->enabled = FALSE;
-  unit->current_gl_target = 0;
+  unit->enabled_gl_target = 0;
   unit->gl_texture = 0;
+  unit->gl_target = 0;
   unit->is_foreign = FALSE;
   unit->dirty_gl_texture = FALSE;
   unit->matrix_stack = _cogl_matrix_stack_new ();
@@ -107,7 +98,7 @@ texture_unit_free (CoglTextureUnit *unit)
 {
   if (unit->layer)
     cogl_object_unref (unit->layer);
-  _cogl_matrix_stack_destroy (unit->matrix_stack);
+  cogl_object_unref (unit->matrix_stack);
 }
 
 CoglTextureUnit *
@@ -160,23 +151,6 @@ _cogl_set_active_texture_unit (int unit_index)
     }
 }
 
-void
-_cogl_disable_texture_unit (int unit_index)
-{
-  CoglTextureUnit *unit;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  unit = &g_array_index (ctx->texture_units, CoglTextureUnit, unit_index);
-
-  if (unit->enabled)
-    {
-      _cogl_set_active_texture_unit (unit_index);
-      GE (glDisable (unit->current_gl_target));
-      unit->enabled = FALSE;
-    }
-}
-
 /* Note: _cogl_bind_gl_texture_transient conceptually has slightly
  * different semantics to OpenGL's glBindTexture because Cogl never
  * cares about tracking multiple textures bound to different targets
@@ -245,6 +219,7 @@ _cogl_delete_gl_texture (GLuint gl_texture)
       if (unit->gl_texture == gl_texture)
         {
           unit->gl_texture = 0;
+          unit->gl_target = 0;
           unit->dirty_gl_texture = FALSE;
         }
     }
@@ -277,21 +252,45 @@ _cogl_pipeline_texture_storage_change_notify (CoglHandle texture)
        * we continue to check the rest */
     }
 }
+static void
+set_glsl_program (GLuint gl_program)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  if (ctx->current_gl_program != gl_program)
+    {
+      GLenum gl_error;
+
+      while ((gl_error = glGetError ()) != GL_NO_ERROR)
+        ;
+      glUseProgram (gl_program);
+      if (glGetError () == GL_NO_ERROR)
+        ctx->current_gl_program = gl_program;
+      else
+        {
+          GE( glUseProgram (0) );
+          ctx->current_gl_program = 0;
+        }
+    }
+}
 
 void
-_cogl_use_program (GLuint gl_program, CoglPipelineProgramType type)
+_cogl_use_fragment_program (GLuint gl_program, CoglPipelineProgramType type)
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   /* If we're changing program type... */
-  if (type != ctx->current_use_program_type)
+  if (type != ctx->current_fragment_program_type)
     {
       /* ... disable the old type */
-      switch (ctx->current_use_program_type)
+      switch (ctx->current_fragment_program_type)
         {
         case COGL_PIPELINE_PROGRAM_TYPE_GLSL:
-          GE( glUseProgram (0) );
-          ctx->current_gl_program = 0;
+          /* If the program contains a vertex shader then we shouldn't
+             disable it */
+          if (ctx->current_vertex_program_type !=
+              COGL_PIPELINE_PROGRAM_TYPE_GLSL)
+            set_glsl_program (0);
           break;
 
         case COGL_PIPELINE_PROGRAM_TYPE_ARBFP:
@@ -323,40 +322,88 @@ _cogl_use_program (GLuint gl_program, CoglPipelineProgramType type)
 
   if (type == COGL_PIPELINE_PROGRAM_TYPE_GLSL)
     {
-#ifdef COGL_PIPELINE_BACKEND_GLSL
-
-      if (ctx->current_gl_program != gl_program)
-        {
-          GLenum gl_error;
-
-          while ((gl_error = glGetError ()) != GL_NO_ERROR)
-            ;
-          glUseProgram (gl_program);
-          if (glGetError () == GL_NO_ERROR)
-            ctx->current_gl_program = gl_program;
-          else
-            {
-              GE( glUseProgram (0) );
-              ctx->current_gl_program = 0;
-            }
-        }
+#ifdef COGL_PIPELINE_FRAGEND_GLSL
+      set_glsl_program (gl_program);
 
 #else
 
-      g_warning ("Unexpected use of GLSL backend!");
+      g_warning ("Unexpected use of GLSL fragend!");
 
-#endif /* COGL_PIPELINE_BACKEND_GLSL */
+#endif /* COGL_PIPELINE_FRAGEND_GLSL */
     }
-#ifndef COGL_PIPELINE_BACKEND_ARBFP
+#ifndef COGL_PIPELINE_FRAGEND_ARBFP
   else if (type == COGL_PIPELINE_PROGRAM_TYPE_ARBFP)
-      g_warning ("Unexpected use of ARBFP backend!");
-#endif /* COGL_PIPELINE_BACKEND_ARBFP */
+    g_warning ("Unexpected use of ARBFP fragend!");
+#endif /* COGL_PIPELINE_FRAGEND_ARBFP */
 
-  ctx->current_use_program_type = type;
+  ctx->current_fragment_program_type = type;
 }
 
-#if defined (COGL_PIPELINE_BACKEND_GLSL) || \
-    defined (COGL_PIPELINE_BACKEND_ARBFP)
+void
+_cogl_use_vertex_program (GLuint gl_program, CoglPipelineProgramType type)
+{
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  /* If we're changing program type... */
+  if (type != ctx->current_vertex_program_type)
+    {
+      /* ... disable the old type */
+      switch (ctx->current_vertex_program_type)
+        {
+        case COGL_PIPELINE_PROGRAM_TYPE_GLSL:
+          /* If the program contains a fragment shader then we shouldn't
+             disable it */
+          if (ctx->current_fragment_program_type !=
+              COGL_PIPELINE_PROGRAM_TYPE_GLSL)
+            set_glsl_program (0);
+          break;
+
+        case COGL_PIPELINE_PROGRAM_TYPE_ARBFP:
+          /* It doesn't make sense to enable ARBfp for the vertex program */
+          g_assert_not_reached ();
+          break;
+
+        case COGL_PIPELINE_PROGRAM_TYPE_FIXED:
+          /* don't need to to anything */
+          break;
+        }
+
+      /* ... and enable the new type */
+      switch (type)
+        {
+        case COGL_PIPELINE_PROGRAM_TYPE_ARBFP:
+          /* It doesn't make sense to enable ARBfp for the vertex program */
+          g_assert_not_reached ();
+          break;
+
+        case COGL_PIPELINE_PROGRAM_TYPE_GLSL:
+        case COGL_PIPELINE_PROGRAM_TYPE_FIXED:
+          /* don't need to to anything */
+          break;
+        }
+    }
+
+  if (type == COGL_PIPELINE_PROGRAM_TYPE_GLSL)
+    {
+#ifdef COGL_PIPELINE_VERTEND_GLSL
+      set_glsl_program (gl_program);
+
+#else
+
+      g_warning ("Unexpected use of GLSL vertend!");
+
+#endif /* COGL_PIPELINE_VERTEND_GLSL */
+    }
+#ifndef COGL_PIPELINE_VERTEND_ARBFP
+  else if (type == COGL_PIPELINE_PROGRAM_TYPE_ARBFP)
+    g_warning ("Unexpected use of ARBFP vertend!");
+#endif /* COGL_PIPELINE_VERTEND_ARBFP */
+
+  ctx->current_vertex_program_type = type;
+}
+
+#if defined (COGL_PIPELINE_FRAGEND_GLSL) || \
+    defined (COGL_PIPELINE_FRAGEND_ARBFP)
 int
 _cogl_get_max_texture_image_units (void)
 {
@@ -431,6 +478,8 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
 {
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
+  /* On GLES2 we'll flush the color later */
+#ifndef HAVE_COGL_GLES2
   if (!skip_gl_color)
     {
       if ((pipelines_difference & COGL_PIPELINE_STATE_COLOR) ||
@@ -446,21 +495,7 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
                           cogl_color_get_alpha_byte (&authority->color)));
         }
     }
-
-  if (pipelines_difference & COGL_PIPELINE_STATE_LIGHTING)
-    {
-      CoglPipeline *authority =
-        _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_LIGHTING);
-      CoglPipelineLightingState *lighting_state =
-        &authority->big_state->lighting_state;
-
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, lighting_state->ambient));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, lighting_state->diffuse));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, lighting_state->specular));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, lighting_state->emission));
-      GE (glMaterialfv (GL_FRONT_AND_BACK, GL_SHININESS,
-                        &lighting_state->shininess));
-    }
+#endif
 
   if (pipelines_difference & COGL_PIPELINE_STATE_BLEND)
     {
@@ -560,18 +595,6 @@ _cogl_pipeline_flush_color_blend_alpha_depth_state (
         }
     }
 
-  if (pipelines_difference & COGL_PIPELINE_STATE_POINT_SIZE)
-    {
-      CoglPipeline *authority =
-        _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_POINT_SIZE);
-
-      if (ctx->point_size_cache != authority->big_state->point_size)
-        {
-          GE( glPointSize (authority->big_state->point_size) );
-          ctx->point_size_cache = authority->big_state->point_size;
-        }
-    }
-
   if (pipeline->real_blend_enable != ctx->gl_blend_enable_cache)
     {
       if (pipeline->real_blend_enable)
@@ -591,7 +614,7 @@ get_max_activateable_texture_units (void)
 
   if (G_UNLIKELY (ctx->max_activateable_texture_units == -1))
     {
-#ifdef HAVE_COGL_GL
+#if defined (HAVE_COGL_GL)
       GLint max_tex_coords;
       GLint max_combined_tex_units;
       GE (glGetIntegerv (GL_MAX_TEXTURE_COORDS, &max_tex_coords));
@@ -599,6 +622,16 @@ get_max_activateable_texture_units (void)
                          &max_combined_tex_units));
       ctx->max_activateable_texture_units =
         MAX (max_tex_coords - 1, max_combined_tex_units);
+#elif defined (HAVE_COGL_GLES2)
+      GLint max_vertex_attribs;
+      GLint max_combined_tex_units;
+      GE (glGetIntegerv (GL_MAX_VERTEX_ATTRIBS, &max_vertex_attribs));
+      GE (glGetIntegerv (GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS,
+                         &max_combined_tex_units));
+      /* Two of the vertex attribs need to be used for the position
+         and color */
+      ctx->max_activateable_texture_units =
+        MAX (max_vertex_attribs - 2, max_combined_tex_units);
 #else
       GE (glGetIntegerv (GL_MAX_TEXTURE_UNITS,
                          &ctx->max_activateable_texture_units));
@@ -692,55 +725,17 @@ flush_layers_common_gl_state_cb (CoglPipelineLayer *layer, void *user_data)
           else
             GE (glBindTexture (gl_target, gl_texture));
           unit->gl_texture = gl_texture;
+          unit->gl_target = gl_target;
         }
 
       unit->is_foreign = _cogl_texture_is_foreign (texture);
 
-      /* Disable the previous target if it was different and it's
-       * still enabled */
-      if (unit->enabled && unit->current_gl_target != gl_target)
-        GE (glDisable (unit->current_gl_target));
-
-      if (!G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_TEXTURING) &&
-          (!unit->enabled || unit->current_gl_target != gl_target))
-        {
-          GE (glEnable (gl_target));
-          unit->enabled = TRUE;
-          unit->current_gl_target = gl_target;
-        }
-
       /* The texture_storage_changed boolean indicates if the
        * CoglTexture's underlying GL texture storage has changed since
        * it was flushed to the texture unit. We've just flushed the
        * latest state so we can reset this. */
       unit->texture_storage_changed = FALSE;
     }
-  else
-    {
-      /* Even though there may be no difference between the last flushed
-       * texture state and the current layers texture state it may be that the
-       * texture unit has been disabled for some time so we need to assert that
-       * it's enabled now.
-       */
-      if (!G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_TEXTURING) &&
-          !unit->enabled)
-        {
-          GE (glEnable (unit->current_gl_target));
-          unit->enabled = TRUE;
-        }
-    }
-
-  if (layers_difference & COGL_PIPELINE_LAYER_STATE_USER_MATRIX)
-    {
-      CoglPipelineLayerState state = COGL_PIPELINE_LAYER_STATE_USER_MATRIX;
-      CoglPipelineLayer *authority =
-        _cogl_pipeline_layer_get_authority (layer, state);
-
-      _cogl_matrix_stack_set (unit->matrix_stack,
-                              &authority->big_state->matrix);
-
-      _cogl_matrix_stack_flush_to_gl (unit->matrix_stack, COGL_MATRIX_TEXTURE);
-    }
 
   /* Under GLES2 the fragment shader will use gl_PointCoord instead of
      replacing the texture coordinates */
@@ -790,10 +785,6 @@ _cogl_pipeline_flush_common_gl_state (CoglPipeline  *pipeline,
   _cogl_pipeline_foreach_layer_internal (pipeline,
                                          flush_layers_common_gl_state_cb,
                                          &state);
-
-  /* Disable additional texture units that may have previously been in use.. */
-  for (; state.i < ctx->texture_units->len; state.i++)
-    _cogl_disable_texture_unit (state.i);
 }
 
 /* Re-assert the layer's wrap modes on the given CoglTexture.
@@ -868,19 +859,20 @@ foreach_texture_unit_update_filter_and_wrap_modes (void)
       CoglTextureUnit *unit =
         &g_array_index (ctx->texture_units, CoglTextureUnit, i);
 
-      if (!unit->enabled)
-        break;
-
       if (unit->layer)
         {
           CoglHandle texture = _cogl_pipeline_layer_get_texture (unit->layer);
-          CoglPipelineFilter min;
-          CoglPipelineFilter mag;
 
-          _cogl_pipeline_layer_get_filters (unit->layer, &min, &mag);
-          _cogl_texture_set_filters (texture, min, mag);
+          if (texture != COGL_INVALID_HANDLE)
+            {
+              CoglPipelineFilter min;
+              CoglPipelineFilter mag;
 
-          _cogl_pipeline_layer_forward_wrap_modes (unit->layer, texture);
+              _cogl_pipeline_layer_get_filters (unit->layer, &min, &mag);
+              _cogl_texture_set_filters (texture, min, mag);
+
+              _cogl_pipeline_layer_forward_wrap_modes (unit->layer, texture);
+            }
         }
     }
 }
@@ -926,46 +918,64 @@ compare_layer_differences_cb (CoglPipelineLayer *layer, void *user_data)
 
 typedef struct
 {
-  const CoglPipelineBackend *backend;
+  const CoglPipelineFragend *fragend;
   CoglPipeline *pipeline;
   unsigned long *layer_differences;
   gboolean error_adding_layer;
   gboolean added_layer;
-} CoglPipelineBackendAddLayerState;
+} CoglPipelineFragendAddLayerState;
 
 
 static gboolean
-backend_add_layer_cb (CoglPipelineLayer *layer,
+fragend_add_layer_cb (CoglPipelineLayer *layer,
                       void *user_data)
 {
-  CoglPipelineBackendAddLayerState *state = user_data;
-  const CoglPipelineBackend *backend = state->backend;
+  CoglPipelineFragendAddLayerState *state = user_data;
+  const CoglPipelineFragend *fragend = state->fragend;
   CoglPipeline *pipeline = state->pipeline;
   int unit_index = _cogl_pipeline_layer_get_unit_index (layer);
-  CoglTextureUnit *unit = _cogl_get_texture_unit (unit_index);
 
   _COGL_GET_CONTEXT (ctx, FALSE);
 
-  /* NB: We don't support the random disabling of texture
-   * units, so as soon as we hit a disabled unit we know all
-   * subsequent units are also disabled */
-  if (!unit->enabled)
-    return FALSE;
-
-  if (G_UNLIKELY (unit_index >= backend->get_max_texture_units ()))
+  /* Either generate per layer code snippets or setup the
+   * fixed function glTexEnv for each layer... */
+  if (G_LIKELY (fragend->add_layer (pipeline,
+                                    layer,
+                                    state->layer_differences[unit_index])))
+    state->added_layer = TRUE;
+  else
     {
-      int j;
-      for (j = unit_index; j < ctx->texture_units->len; j++)
-        _cogl_disable_texture_unit (j);
-      /* TODO: although this isn't considered an error that
-       * warrants falling back to a different backend we
-       * should print a warning here. */
+      state->error_adding_layer = TRUE;
       return FALSE;
     }
 
-  /* Either generate per layer code snippets or setup the
-   * fixed function glTexEnv for each layer... */
-  if (G_LIKELY (backend->add_layer (pipeline,
+  return TRUE;
+}
+
+typedef struct
+{
+  const CoglPipelineVertend *vertend;
+  CoglPipeline *pipeline;
+  unsigned long *layer_differences;
+  gboolean error_adding_layer;
+  gboolean added_layer;
+} CoglPipelineVertendAddLayerState;
+
+
+static gboolean
+vertend_add_layer_cb (CoglPipelineLayer *layer,
+                      void *user_data)
+{
+  CoglPipelineVertendAddLayerState *state = user_data;
+  const CoglPipelineVertend *vertend = state->vertend;
+  CoglPipeline *pipeline = state->pipeline;
+  int unit_index = _cogl_pipeline_layer_get_unit_index (layer);
+
+  _COGL_GET_CONTEXT (ctx, FALSE);
+
+  /* Either enerate per layer code snippets or setup the
+   * fixed function matrix uniforms for each layer... */
+  if (G_LIKELY (vertend->add_layer (pipeline,
                                     layer,
                                     state->layer_differences[unit_index])))
     state->added_layer = TRUE;
@@ -1053,7 +1063,8 @@ _cogl_pipeline_flush_gl_state (CoglPipeline *pipeline,
     {
       /* Bail out asap if we've been asked to re-flush the already current
        * pipeline and we can see the pipeline hasn't changed */
-      if (ctx->current_pipeline_age == pipeline->age)
+      if (ctx->current_pipeline_age == pipeline->age &&
+          ctx->current_pipeline_skip_gl_color == skip_gl_color)
         goto done;
 
       pipelines_difference = ctx->current_pipeline_changes_since_flush;
@@ -1095,8 +1106,6 @@ _cogl_pipeline_flush_gl_state (CoglPipeline *pipeline,
    * 2) then foreach layer:
    *  determine gl_target/gl_texture
    *  bind texture
-   *  enable/disable target
-   *  flush user matrix
    *
    *  Note: After _cogl_pipeline_flush_common_gl_state you can expect
    *  all state of the layers corresponding texture unit to be
@@ -1114,59 +1123,110 @@ _cogl_pipeline_flush_gl_state (CoglPipeline *pipeline,
    * configuration and in that case it will report an error and we
    * will fallback to a different backend.
    *
-   * NB: if pipeline->backend != COGL_PIPELINE_BACKEND_UNDEFINED then
+   * NB: if pipeline->backend != COGL_PIPELINE_FRAGEND_UNDEFINED then
    * we have previously managed to successfully flush this pipeline
    * with the given backend so we will simply use that to avoid
    * fallback code paths.
    */
 
-  if (pipeline->backend == COGL_PIPELINE_BACKEND_UNDEFINED)
-    _cogl_pipeline_set_backend (pipeline, COGL_PIPELINE_BACKEND_DEFAULT);
+  if (pipeline->fragend == COGL_PIPELINE_FRAGEND_UNDEFINED)
+    _cogl_pipeline_set_fragend (pipeline, COGL_PIPELINE_FRAGEND_DEFAULT);
 
-  for (i = pipeline->backend;
-       i < G_N_ELEMENTS (_cogl_pipeline_backends);
-       i++, _cogl_pipeline_set_backend (pipeline, i))
+  for (i = pipeline->fragend;
+       i < G_N_ELEMENTS (_cogl_pipeline_fragends);
+       i++, _cogl_pipeline_set_fragend (pipeline, i))
     {
-      const CoglPipelineBackend *backend = _cogl_pipeline_backends[i];
-      CoglPipelineBackendAddLayerState state;
+      const CoglPipelineFragend *fragend = _cogl_pipeline_fragends[i];
+      CoglPipelineFragendAddLayerState state;
 
-      /* E.g. For backends generating code they can setup their
+      /* E.g. For fragends generating code they can setup their
        * scratch buffers here... */
-      if (G_UNLIKELY (!backend->start (pipeline,
+      if (G_UNLIKELY (!fragend->start (pipeline,
                                        n_layers,
                                        pipelines_difference,
                                        n_tex_coord_attribs)))
         continue;
 
-      state.backend = backend;
+      state.fragend = fragend;
       state.pipeline = pipeline;
       state.layer_differences = layer_differences;
       state.error_adding_layer = FALSE;
       state.added_layer = FALSE;
       _cogl_pipeline_foreach_layer_internal (pipeline,
-                                             backend_add_layer_cb,
+                                             fragend_add_layer_cb,
                                              &state);
 
       if (G_UNLIKELY (state.error_adding_layer))
         continue;
 
       if (!state.added_layer &&
-          backend->passthrough &&
-          G_UNLIKELY (!backend->passthrough (pipeline)))
+          fragend->passthrough &&
+          G_UNLIKELY (!fragend->passthrough (pipeline)))
         continue;
 
-      /* For backends generating code they may compile and link their
+      /* For fragends generating code they may compile and link their
        * programs here, update any uniforms and tell OpenGL to use
        * that program.
        */
-      if (G_UNLIKELY (!backend->end (pipeline, pipelines_difference)))
+      if (G_UNLIKELY (!fragend->end (pipeline, pipelines_difference)))
         continue;
 
       break;
     }
 
-  if (G_UNLIKELY (i >= G_N_ELEMENTS (_cogl_pipeline_backends)))
-    g_warning ("No usable pipeline backend was found!");
+  if (G_UNLIKELY (i >= G_N_ELEMENTS (_cogl_pipeline_fragends)))
+    g_warning ("No usable pipeline fragment backend was found!");
+
+  /* Now flush the vertex processing state according to the current
+   * vertex processing backend.
+   */
+
+  if (pipeline->vertend == COGL_PIPELINE_VERTEND_UNDEFINED)
+    _cogl_pipeline_set_vertend (pipeline, COGL_PIPELINE_VERTEND_DEFAULT);
+
+  for (i = pipeline->vertend;
+       i < G_N_ELEMENTS (_cogl_pipeline_vertends);
+       i++, _cogl_pipeline_set_vertend (pipeline, i))
+    {
+      const CoglPipelineVertend *vertend = _cogl_pipeline_vertends[i];
+      CoglPipelineVertendAddLayerState state;
+
+      /* E.g. For vertends generating code they can setup their
+       * scratch buffers here... */
+      if (G_UNLIKELY (!vertend->start (pipeline,
+                                       n_layers,
+                                       pipelines_difference)))
+        continue;
+
+      state.vertend = vertend;
+      state.pipeline = pipeline;
+      state.layer_differences = layer_differences;
+      state.error_adding_layer = FALSE;
+      state.added_layer = FALSE;
+      _cogl_pipeline_foreach_layer_internal (pipeline,
+                                             vertend_add_layer_cb,
+                                             &state);
+
+      if (G_UNLIKELY (state.error_adding_layer))
+        continue;
+
+      /* For vertends generating code they may compile and link their
+       * programs here, update any uniforms and tell OpenGL to use
+       * that program.
+       */
+      if (G_UNLIKELY (!vertend->end (pipeline, pipelines_difference)))
+        continue;
+
+      break;
+    }
+
+  if (G_UNLIKELY (i >= G_N_ELEMENTS (_cogl_pipeline_vertends)))
+    g_warning ("No usable pipeline vertex backend was found!");
+
+  for (i = 0; i < COGL_PIPELINE_N_PROGENDS; i++)
+    if (_cogl_pipeline_progends[i]->end)
+      _cogl_pipeline_progends[i]->end (pipeline, pipelines_difference,
+                                       n_tex_coord_attribs);
 
   /* FIXME: This reference is actually resulting in lots of
    * copy-on-write reparenting because one-shot pipelines end up
@@ -1186,6 +1246,34 @@ _cogl_pipeline_flush_gl_state (CoglPipeline *pipeline,
 
 done:
 
+  /* We can't assume the color will be retained between flushes on
+     GLES2 because the generic attribute values are not stored as part
+     of the program object so they could be overridden by any
+     attribute changes in another program */
+#ifdef HAVE_COGL_GLES2
+  if (!skip_gl_color)
+    {
+      int attribute;
+      CoglPipeline *authority =
+        _cogl_pipeline_get_authority (pipeline, COGL_PIPELINE_STATE_COLOR);
+
+      attribute = _cogl_pipeline_progend_glsl_get_color_attribute (pipeline);
+      if (attribute != -1)
+        GE (glVertexAttrib4f (attribute,
+                              cogl_color_get_red_float (&authority->color),
+                              cogl_color_get_green_float (&authority->color),
+                              cogl_color_get_blue_float (&authority->color),
+                              cogl_color_get_alpha_float (&authority->color)));
+    }
+#endif
+
+  /* Give any progends a chance to update any uniforms that might not
+     depend on the material state. This is used on GLES2 to update the
+     matrices */
+  for (i = 0; i < COGL_PIPELINE_N_PROGENDS; i++)
+    if (_cogl_pipeline_progends[i]->pre_paint)
+      _cogl_pipeline_progends[i]->pre_paint (pipeline);
+
   /* Handle the fact that OpenGL associates texture filter and wrap
    * modes with the texture objects not the texture units... */
   foreach_texture_unit_update_filter_and_wrap_modes ();
@@ -1199,10 +1287,10 @@ done:
    * _cogl_bind_gl_texture_transient)
    */
   unit1 = _cogl_get_texture_unit (1);
-  if (unit1->enabled && unit1->dirty_gl_texture)
+  if (cogl_pipeline_get_n_layers (pipeline) > 1 && unit1->dirty_gl_texture)
     {
       _cogl_set_active_texture_unit (1);
-      GE (glBindTexture (unit1->current_gl_target, unit1->gl_texture));
+      GE (glBindTexture (unit1->gl_target, unit1->gl_texture));
       unit1->dirty_gl_texture = FALSE;
     }
 
