@@ -29,17 +29,33 @@
 
 #define __COGL_H_INSIDE__
 
+#ifdef COGL_ENABLE_EXPERIMENTAL_2_0_API
+#ifndef COGL_ENABLE_EXPERIMENTAL_API
+#define COGL_ENABLE_EXPERIMENTAL_API
+#endif
+#endif
+
+/* We currently keep gtype integration delimited in case we eventually
+ * want to split it out into a separate utility library when Cogl
+ * becomes a standalone project. (like cairo-gobject.so)
+ */
+#define _COGL_SUPPORTS_GTYPE_INTEGRATION
+
 #include <cogl/cogl-defines.h>
 
 #include <cogl/cogl-object.h>
 #include <cogl/cogl-bitmap.h>
 #include <cogl/cogl-color.h>
 #include <cogl/cogl-fixed.h>
-#include <cogl/cogl-material.h>
+#include <cogl/cogl-material-compat.h>
 #include <cogl/cogl-matrix.h>
 #include <cogl/cogl-offscreen.h>
 #include <cogl/cogl-primitives.h>
+#ifdef COGL_ENABLE_EXPERIMENTAL_2_0_API
+#include <cogl/cogl2-path.h>
+#else
 #include <cogl/cogl-path.h>
+#endif
 #include <cogl/cogl-shader.h>
 #include <cogl/cogl-texture.h>
 #include <cogl/cogl-types.h>
@@ -54,6 +70,12 @@
 #include <cogl/cogl-pixel-array.h>
 #include <cogl/cogl-vector.h>
 #include <cogl/cogl-texture-3d.h>
+#include <cogl/cogl-index-array.h>
+#include <cogl/cogl-vertex-array.h>
+#include <cogl/cogl-indices.h>
+#include <cogl/cogl-vertex-attribute.h>
+#include <cogl/cogl-primitive.h>
+#include <cogl/cogl-pipeline.h>
 #endif
 
 G_BEGIN_DECLS
@@ -574,19 +596,61 @@ cogl_clear (const CoglColor *color,
 
 /**
  * cogl_set_source:
- * @material: A #CoglHandle for a material
+ * @material: A #CoglMaterial
  *
- * This function sets the source material that will be used to fill subsequent
- * geometry emitted via the cogl API.
- *
- * <note>In the future we may add the ability to set a front facing material,
- * and a back facing material, in which case this function will set both to the
- * same.</note>
+ * This function changes the material at the top of the source stack.
+ * The material at the top of this stack defines the GPU state used to
+ * process subsequent primitives, such as rectangles drawn with
+ * cogl_rectangle() or vertices drawn using cogl_vertex_buffer_draw().
  *
  * Since: 1.0
  */
 void
-cogl_set_source (CoglHandle material);
+cogl_set_source (void *material);
+
+/**
+ * cogl_get_source:
+ *
+ * Returns the current source material as previously set using
+ * cogl_set_source().
+ *
+ * <note>You should typically consider the returned material immutable
+ * and not try to change any of its properties unless you own a
+ * reference to that material. At times you may be able to get a
+ * reference to an internally managed materials and the result of
+ * modifying such materials is undefined.</note>
+ *
+ * Return value: The current source material.
+ *
+ * Since: 1.6
+ */
+void *
+cogl_get_source (void);
+
+/**
+ * cogl_push_source:
+ * @material: A #CoglMaterial
+ *
+ * Pushes the given @material to the top of the source stack. The
+ * material at the top of this stack defines the GPU state used to
+ * process later primitives as defined by cogl_set_source().
+ *
+ * Since: 1.6
+ */
+void
+cogl_push_source (void *material);
+
+/**
+ * cogl_pop_source:
+ *
+ * Removes the material at the top of the source stack. The material
+ * at the top of this stack defines the GPU state used to process
+ * later primitives as defined by cogl_set_source().
+ *
+ * Since: 1.6
+ */
+void
+cogl_pop_source (void);
 
 /**
  * cogl_set_source_color:
